"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UpdateInstructionsVisitor = void 0;
const nodes = __importStar(require("../../nodes"));
const utils_1 = require("../../utils");
const TransformNodesVisitor_1 = require("./TransformNodesVisitor");
class UpdateInstructionsVisitor extends TransformNodesVisitor_1.TransformNodesVisitor {
    constructor(map) {
        const transforms = Object.entries(map).map(([selector, updates]) => {
            const selectorStack = selector.split('.');
            const name = selectorStack.pop();
            return {
                selector: { type: 'InstructionNode', stack: selectorStack, name },
                transformer: (node, stack, program) => {
                    var _a;
                    nodes.assertInstructionNode(node);
                    if (typeof updates === 'function') {
                        return updates(node, stack, program);
                    }
                    if ('delete' in updates) {
                        return null;
                    }
                    const { accounts: accountUpdates, args: argsUpdates, ...metadataUpdates } = updates;
                    const newName = (0, utils_1.mainCase)((_a = updates.name) !== null && _a !== void 0 ? _a : node.name);
                    const { newArgs, newExtraArgs, newArgDefaults } = this.handleInstructionArgs(node, newName, argsUpdates !== null && argsUpdates !== void 0 ? argsUpdates : {});
                    const newMetadata = {
                        ...node.metadata,
                        ...this.handleMetadata(metadataUpdates),
                        argDefaults: newArgDefaults,
                    };
                    const newAccounts = node.accounts.map((account) => this.handleInstructionAccount(account, accountUpdates !== null && accountUpdates !== void 0 ? accountUpdates : {}));
                    return new nodes.InstructionNode(newMetadata, newAccounts, newArgs, newExtraArgs, node.subInstructions);
                },
            };
        });
        super(transforms);
        this.map = map;
        this.allAccounts = new Map();
    }
    visitRoot(root) {
        root.allAccounts.forEach((account) => {
            this.allAccounts.set(account.name, account);
        });
        return super.visitRoot(root);
    }
    handleMetadata(metadataUpdates) {
        const metadata = metadataUpdates;
        if (metadataUpdates.bytesCreatedOnChain) {
            if (metadataUpdates.bytesCreatedOnChain.kind === 'account') {
                metadata.bytesCreatedOnChain = {
                    includeHeader: true,
                    importFrom: 'generated',
                    ...metadataUpdates.bytesCreatedOnChain,
                };
            }
            else if (metadataUpdates.bytesCreatedOnChain.kind === 'resolver') {
                metadata.bytesCreatedOnChain = {
                    importFrom: 'hooked',
                    ...metadataUpdates.bytesCreatedOnChain,
                };
            }
            else {
                metadata.bytesCreatedOnChain = {
                    includeHeader: true,
                    ...metadataUpdates.bytesCreatedOnChain,
                };
            }
        }
        return metadata;
    }
    handleInstructionAccount(account, accountUpdates) {
        var _a, _b, _c, _d, _e, _f;
        const accountUpdate = accountUpdates === null || accountUpdates === void 0 ? void 0 : accountUpdates[account.name];
        if (((_a = accountUpdate === null || accountUpdate === void 0 ? void 0 : accountUpdate.defaultsTo) === null || _a === void 0 ? void 0 : _a.kind) === 'pda') {
            const pdaAccount = (0, utils_1.mainCase)((_c = (_b = accountUpdate === null || accountUpdate === void 0 ? void 0 : accountUpdate.defaultsTo) === null || _b === void 0 ? void 0 : _b.pdaAccount) !== null && _c !== void 0 ? _c : account.name);
            return {
                ...account,
                ...accountUpdate,
                defaultsTo: {
                    pdaAccount,
                    importFrom: 'generated',
                    seeds: (_e = (_d = this.allAccounts.get(pdaAccount)) === null || _d === void 0 ? void 0 : _d.instructionAccountDefaultSeeds) !== null && _e !== void 0 ? _e : {},
                    ...accountUpdate === null || accountUpdate === void 0 ? void 0 : accountUpdate.defaultsTo,
                },
            };
        }
        if (((_f = accountUpdate === null || accountUpdate === void 0 ? void 0 : accountUpdate.defaultsTo) === null || _f === void 0 ? void 0 : _f.kind) === 'resolver') {
            return {
                ...account,
                ...accountUpdate,
                defaultsTo: {
                    importFrom: 'hooked',
                    dependsOn: [],
                    ...accountUpdate === null || accountUpdate === void 0 ? void 0 : accountUpdate.defaultsTo,
                },
            };
        }
        return accountUpdate
            ? { ...account, ...accountUpdate }
            : account;
    }
    handleInstructionArgs(instruction, newInstructionName, argUpdates) {
        const usedArgs = new Set();
        let newArgs = instruction.args;
        if (!nodes.isTypeDefinedLinkNode(instruction.args)) {
            const fields = instruction.args.fields.map((field) => {
                var _a, _b, _c;
                const argUpdate = argUpdates[field.name];
                if (!argUpdate)
                    return field;
                usedArgs.add(field.name);
                return new nodes.TypeStructFieldNode({
                    ...field.metadata,
                    name: (_a = argUpdate.name) !== null && _a !== void 0 ? _a : field.name,
                    docs: (_b = argUpdate.docs) !== null && _b !== void 0 ? _b : field.metadata.docs,
                }, (_c = argUpdate.type) !== null && _c !== void 0 ? _c : field.type);
            });
            newArgs = new nodes.TypeStructNode(`${newInstructionName}InstructionData`, fields);
        }
        let newExtraArgs = instruction.extraArgs;
        if (!nodes.isTypeDefinedLinkNode(instruction.extraArgs)) {
            const fields = instruction.extraArgs.fields.map((field) => {
                var _a, _b, _c;
                if (usedArgs.has(field.name))
                    return field;
                const argUpdate = argUpdates[field.name];
                if (!argUpdate)
                    return field;
                usedArgs.add(field.name);
                return new nodes.TypeStructFieldNode({
                    ...field.metadata,
                    name: (_a = argUpdate.name) !== null && _a !== void 0 ? _a : field.name,
                    docs: (_b = argUpdate.docs) !== null && _b !== void 0 ? _b : field.metadata.docs,
                }, (_c = argUpdate.type) !== null && _c !== void 0 ? _c : field.type);
            });
            const newExtraFields = Object.entries(argUpdates)
                .filter(([argName]) => !usedArgs.has(argName))
                .map(([argName, argUpdate]) => {
                var _a, _b, _c;
                const type = (_a = argUpdate.type) !== null && _a !== void 0 ? _a : null;
                nodes.assertTypeNode(type);
                return new nodes.TypeStructFieldNode({
                    name: (_b = argUpdate.name) !== null && _b !== void 0 ? _b : argName,
                    docs: (_c = argUpdate.docs) !== null && _c !== void 0 ? _c : [],
                    defaultsTo: null,
                }, type);
            });
            fields.push(...newExtraFields);
            newExtraArgs = new nodes.TypeStructNode(`${newInstructionName}InstructionExtra`, fields);
        }
        const newArgDefaults = instruction.metadata.argDefaults;
        Object.entries(argUpdates).forEach(([argName, argUpdate]) => {
            if ((argUpdate === null || argUpdate === void 0 ? void 0 : argUpdate.defaultsTo) === undefined)
                return;
            if (argUpdate.defaultsTo === null) {
                delete newArgDefaults[argName];
            }
            else {
                newArgDefaults[argName] = this.parseDefaultArg(argUpdate.defaultsTo);
            }
        });
        return { newArgs, newExtraArgs, newArgDefaults };
    }
    parseDefaultArg(argDefault) {
        if ((argDefault === null || argDefault === void 0 ? void 0 : argDefault.kind) === 'resolver') {
            return { importFrom: 'hooked', dependsOn: [], ...argDefault };
        }
        return argDefault;
    }
}
exports.UpdateInstructionsVisitor = UpdateInstructionsVisitor;
//# sourceMappingURL=UpdateInstructionsVisitor.js.map