import * as nodes from '../../nodes';
import { ImportFrom } from '../ImportFrom';
import { InstructionNodeAccountDefaultsInput } from './SetInstructionAccountDefaultValuesVisitor';
import { NodeTransformer, TransformNodesVisitor } from './TransformNodesVisitor';
export type InstructionUpdates = NodeTransformer<nodes.InstructionNode> | {
    delete: true;
} | (InstructionMetadataUpdates & {
    accounts?: InstructionAccountUpdates;
    args?: InstructionArgUpdates;
});
export type InstructionMetadataUpdates = Partial<Omit<nodes.InstructionNodeMetadata, 'bytesCreatedOnChain'> & {
    bytesCreatedOnChain: InstructionNodeBytesCreatedOnChainInput | null;
}>;
export type InstructionAccountUpdates = Record<string, Partial<Omit<nodes.InstructionNodeAccount, 'defaultsTo'> & {
    defaultsTo: InstructionNodeAccountDefaultsInput;
}>>;
export type InstructionNodeArgDefaultsInput = nodes.InstructionNodeArgDefaults | {
    kind: 'resolver';
    name: string;
    importFrom?: ImportFrom;
    dependsOn?: nodes.InstructionNodeInputDependency[];
};
export type InstructionArgUpdates = Record<string, {
    name?: string;
    docs?: string[];
    type?: nodes.TypeNode;
    defaultsTo?: nodes.InstructionNodeArgDefaults | null;
}>;
type InstructionNodeBytesCreatedOnChainInput = {
    kind: 'number';
    value: number;
    includeHeader?: boolean;
} | {
    kind: 'arg';
    name: string;
    includeHeader?: boolean;
} | {
    kind: 'account';
    name: string;
    importFrom?: ImportFrom;
    includeHeader?: boolean;
} | {
    kind: 'resolver';
    name: string;
    importFrom?: ImportFrom;
};
export declare class UpdateInstructionsVisitor extends TransformNodesVisitor {
    readonly map: Record<string, InstructionUpdates>;
    protected allAccounts: Map<string, nodes.AccountNode>;
    constructor(map: Record<string, InstructionUpdates>);
    visitRoot(root: nodes.RootNode): nodes.Node | null;
    handleMetadata(metadataUpdates: InstructionMetadataUpdates): Partial<nodes.InstructionNodeMetadata>;
    handleInstructionAccount(account: nodes.InstructionNodeAccount, accountUpdates: InstructionAccountUpdates): nodes.InstructionNodeAccount;
    handleInstructionArgs(instruction: nodes.InstructionNode, newInstructionName: string, argUpdates: InstructionArgUpdates): {
        newArgs: nodes.TypeStructNode | nodes.TypeDefinedLinkNode;
        newExtraArgs: nodes.TypeStructNode | nodes.TypeDefinedLinkNode;
        newArgDefaults: Record<string, nodes.InstructionNodeArgDefaults>;
    };
    parseDefaultArg(argDefault: InstructionNodeArgDefaultsInput): nodes.InstructionNodeArgDefaults;
}
export {};
